@@ -7,6 +7,8 @@ mod column;
 mod count;
 mod element;
 mod eval;
+#[cfg(feature = "dtype-struct")]
+mod field;
 mod filter;
 mod gather;
 mod group_iter;
@@ -16,6 +18,8 @@ mod rolling;
 mod slice;
 mod sort;
 mod sortby;
+#[cfg(feature = "dtype-struct")]
+mod structeval;
 mod ternary;
 mod window;
 
@@ -34,6 +38,8 @@ pub(crate) use column::*;
 pub(crate) use count::*;
 pub(crate) use element::*;
 pub(crate) use eval::*;
+#[cfg(feature = "dtype-struct")]
+pub(crate) use field::*;
 pub(crate) use filter::*;
 pub(crate) use gather::*;
 pub(crate) use literal::*;
@@ -45,6 +51,8 @@ pub(crate) use rolling::RollingExpr;
 pub(crate) use slice::*;
 pub(crate) use sort::*;
 pub(crate) use sortby::*;
+#[cfg(feature = "dtype-struct")]
+pub(crate) use structeval::*;
 pub(crate) use ternary::*;
 pub use window::window_function_format_order_by;
 pub(crate) use window::*;
