@@ -4,11 +4,11 @@ use std::ops::{Deref, DerefMut};
 #[cfg(feature = "cloud")]
 pub use async_writeable::AsyncWriteable;
 use polars_core::config;
-use polars_error::{PolarsError, PolarsResult, feature_gated};
+use polars_error::{PolarsError, PolarsResult, feature_gated, polars_err};
 use polars_utils::create_file;
 use polars_utils::file::close_file;
 use polars_utils::mmap::ensure_not_mapped;
-use polars_utils::plpath::PlPathRef;
+use polars_utils::pl_path::{PlRefPath, format_file_uri};
 
 use super::sync_on_close::SyncOnCloseType;
 use crate::cloud::CloudOptions;
@@ -38,60 +38,45 @@ pub enum Writeable {
 
 impl Writeable {
     pub fn try_new(
-        path: PlPathRef<'_>,
-        #[cfg_attr(not(feature = "cloud"), allow(unused))] cloud_options: Option<&CloudOptions>,
-        #[cfg_attr(not(feature = "cloud"), allow(unused))] cloud_upload_chunk_size: usize,
+        path: PlRefPath,
+        #[cfg_attr(not(feature = "cloud"), expect(unused))] cloud_options: Option<&CloudOptions>,
+        #[cfg_attr(not(feature = "cloud"), expect(unused))] cloud_upload_chunk_size: usize,
     ) -> PolarsResult<Self> {
-        match path {
-            PlPathRef::Cloud(_) => {
-                feature_gated!("cloud", {
-                    use crate::cloud::BlockingCloudWriter;
+        Ok(if path.has_scheme() {
+            feature_gated!("cloud", {
+                use crate::cloud::BlockingCloudWriter;
 
-                    let writer = crate::pl_async::get_runtime().block_in_place_on(
-                        BlockingCloudWriter::new(path, cloud_options, cloud_upload_chunk_size),
-                    )?;
+                let writer = crate::pl_async::get_runtime().block_in_place_on(
+                    BlockingCloudWriter::new(path, cloud_options, cloud_upload_chunk_size),
+                )?;
 
-                    Ok(Self::Cloud(writer))
-                })
-            },
-            PlPathRef::Local(path) if config::force_async() => {
-                feature_gated!("cloud", {
-                    use crate::cloud::BlockingCloudWriter;
+                Self::Cloud(writer)
+            })
+        } else if config::force_async() {
+            feature_gated!("cloud", {
+                use crate::cloud::BlockingCloudWriter;
 
-                    let path = resolve_homedir(&path);
-
-                    create_file(&path)?;
-                    let path = std::fs::canonicalize(&path)?;
-
-                    ensure_not_mapped(&path.metadata()?)?;
-
-                    let path = format!(
-                        "file://{}",
-                        if cfg!(target_family = "windows") {
-                            path.to_str().unwrap().strip_prefix(r#"\\?\"#).unwrap()
-                        } else {
-                            path.to_str().unwrap()
-                        }
-                    );
-
-                    let writer = crate::pl_async::get_runtime().block_in_place_on(
-                        BlockingCloudWriter::new(
-                            PlPathRef::new(&path),
-                            cloud_options,
-                            cloud_upload_chunk_size,
-                        ),
-                    )?;
-
-                    Ok(Self::Cloud(writer))
-                })
-            },
-            PlPathRef::Local(path) => {
-                let path = resolve_homedir(&path);
+                let path = resolve_homedir(path.as_std_path());
                 create_file(&path)?;
+                let path = std::fs::canonicalize(&path)?;
 
-                Ok(Self::Local(polars_utils::open_file_write(&path)?))
-            },
-        }
+                ensure_not_mapped(&path.metadata()?)?;
+
+                let path = path.to_str().ok_or_else(|| polars_err!(non_utf8_path))?;
+                let path = format_file_uri(path);
+
+                let writer = crate::pl_async::get_runtime().block_in_place_on(
+                    BlockingCloudWriter::new(path, cloud_options, cloud_upload_chunk_size),
+                )?;
+
+                Self::Cloud(writer)
+            })
+        } else {
+            let path = resolve_homedir(path.as_std_path());
+            create_file(&path)?;
+
+            Self::Local(polars_utils::open_file_write(&path)?)
+        })
     }
 
     /// This returns `Result<>` - if a write was performed before calling this,
@@ -213,7 +198,7 @@ mod async_writeable {
 
     use polars_error::{PolarsError, PolarsResult};
     use polars_utils::file::close_file;
-    use polars_utils::plpath::PlPathRef;
+    use polars_utils::pl_path::PlRefPath;
     use tokio::io::AsyncWriteExt;
     use tokio::task;
 
@@ -258,7 +243,7 @@ mod async_writeable {
 
     impl AsyncWriteable {
         pub async fn try_new(
-            path: PlPathRef<'_>,
+            path: PlRefPath,
             cloud_options: Option<&CloudOptions>,
             cloud_upload_chunk_size: usize,
         ) -> PolarsResult<Self> {
