@@ -1,7 +1,7 @@
 use polars_io::RowIndex;
 #[cfg(feature = "is_between")]
 use polars_ops::prelude::ClosedInterval;
-use polars_utils::plpath::PlPath;
+use polars_utils::pl_path::PlRefPath;
 use polars_utils::slice_enum::Slice;
 
 use super::*;
@@ -366,7 +366,7 @@ fn test_parquet_globbing() -> PolarsResult<()> {
     let _guard = SINGLE_LOCK.lock().unwrap();
     let glob = "../../examples/datasets/foods*.parquet";
     let df = LazyFrame::scan_parquet(
-        PlPath::new(glob),
+        PlRefPath::new(glob),
         ScanArgsParquet {
             n_rows: None,
             cache: true,
@@ -393,7 +393,7 @@ fn test_scan_parquet_limit_9001() {
         rechunk: true,
         ..Default::default()
     };
-    let q = LazyFrame::scan_parquet(PlPath::new(path), args)
+    let q = LazyFrame::scan_parquet(PlRefPath::new(path), args)
         .unwrap()
         .limit(3);
     let IRPlan {
@@ -418,7 +418,7 @@ fn test_ipc_globbing() -> PolarsResult<()> {
     init_files();
     let glob = "../../examples/datasets/foods*.ipc";
     let df = LazyFrame::scan_ipc(
-        PlPath::new(glob),
+        PlRefPath::new(glob),
         Default::default(),
         UnifiedScanArgs {
             cache: true,
@@ -448,7 +448,9 @@ fn slice_at_union(lp_arena: &Arena<IR>, lp: Node) -> bool {
 #[test]
 fn test_csv_globbing() -> PolarsResult<()> {
     let glob = "../../examples/datasets/foods*.csv";
-    let full_df = LazyCsvReader::new(PlPath::new(glob)).finish()?.collect()?;
+    let full_df = LazyCsvReader::new(PlRefPath::new(glob))
+        .finish()?
+        .collect()?;
 
     // all 5 files * 27 rows
     assert_eq!(full_df.shape(), (135, 4));
@@ -457,13 +459,13 @@ fn test_csv_globbing() -> PolarsResult<()> {
     assert_eq!(cal.get(53)?, AnyValue::Int64(194));
 
     let glob = "../../examples/datasets/foods*.csv";
-    let lf = LazyCsvReader::new(PlPath::new(glob))
+    let lf = LazyCsvReader::new(PlRefPath::new(glob))
         .finish()?
         .slice(0, 100);
 
     let df = lf.clone().collect()?;
     assert_eq!(df, full_df.slice(0, 100));
-    let df = LazyCsvReader::new(PlPath::new(glob))
+    let df = LazyCsvReader::new(PlRefPath::new(glob))
         .finish()?
         .slice(20, 60)
         .collect()?;
@@ -474,7 +476,7 @@ fn test_csv_globbing() -> PolarsResult<()> {
     let node = lf.optimize(&mut lp_arena, &mut expr_arena)?;
     assert!(slice_at_union(&lp_arena, node));
 
-    let lf = LazyCsvReader::new(PlPath::new(glob))
+    let lf = LazyCsvReader::new(PlRefPath::new(glob))
         .finish()?
         .filter(col("sugars_g").lt(lit(1i32)))
         .slice(0, 100);
@@ -490,7 +492,7 @@ fn test_ndjson_globbing() -> PolarsResult<()> {
     // for side effects
     init_files();
     let glob = "../../examples/datasets/foods*.ndjson";
-    let df = LazyJsonLineReader::new(PlPath::new(glob))
+    let df = LazyJsonLineReader::new(PlRefPath::new(glob))
         .finish()?
         .collect()?;
     assert_eq!(df.shape(), (54, 4));
@@ -516,7 +518,7 @@ fn test_union_and_agg_projections() -> PolarsResult<()> {
     // a union vstacks columns and aggscan optimization determines columns to aggregate in a
     // hashmap, if that doesn't set them sorted the vstack will panic.
     let lf1: LazyFrame = DslBuilder::scan_parquet(
-        ScanSources::Paths(FromIterator::from_iter([PlPath::new(GLOB_PARQUET)])),
+        ScanSources::Paths(FromIterator::from_iter([PlRefPath::new(GLOB_PARQUET)])),
         ParquetOptions::default(),
         UnifiedScanArgs {
             extra_columns_policy: ExtraColumnsPolicy::Ignore,
@@ -528,7 +530,7 @@ fn test_union_and_agg_projections() -> PolarsResult<()> {
     .into();
 
     let lf2: LazyFrame = DslBuilder::scan_ipc(
-        ScanSources::Paths(FromIterator::from_iter([PlPath::new(GLOB_IPC)])),
+        ScanSources::Paths(FromIterator::from_iter([PlRefPath::new(GLOB_IPC)])),
         IpcScanOptions {},
         UnifiedScanArgs {
             extra_columns_policy: ExtraColumnsPolicy::Ignore,
@@ -540,7 +542,7 @@ fn test_union_and_agg_projections() -> PolarsResult<()> {
     .into();
 
     let lf3: LazyFrame = DslBuilder::scan_csv(
-        ScanSources::Paths(FromIterator::from_iter([PlPath::new(GLOB_CSV)])),
+        ScanSources::Paths(FromIterator::from_iter([PlRefPath::new(GLOB_CSV)])),
         CsvReadOptions::default(),
         UnifiedScanArgs {
             extra_columns_policy: ExtraColumnsPolicy::Ignore,
@@ -616,7 +618,7 @@ fn test_slice_filter() -> PolarsResult<()> {
 
 #[test]
 fn skip_rows_and_slice() -> PolarsResult<()> {
-    let out = LazyCsvReader::new(PlPath::new(FOODS_CSV))
+    let out = LazyCsvReader::new(PlRefPath::new(FOODS_CSV))
         .with_skip_rows(4)
         .finish()?
         .limit(1)
@@ -630,7 +632,7 @@ fn skip_rows_and_slice() -> PolarsResult<()> {
 fn test_row_index_on_files() -> PolarsResult<()> {
     let _guard = SINGLE_LOCK.lock().unwrap();
     for offset in [0 as IdxSize, 10] {
-        let lf = LazyCsvReader::new(PlPath::new(FOODS_CSV))
+        let lf = LazyCsvReader::new(PlRefPath::new(FOODS_CSV))
             .with_row_index(Some(RowIndex {
                 name: PlSmallStr::from_static("index"),
                 offset,
@@ -645,7 +647,7 @@ fn test_row_index_on_files() -> PolarsResult<()> {
             (offset..27 + offset).collect::<Vec<_>>()
         );
 
-        let lf = LazyFrame::scan_parquet(PlPath::new(FOODS_PARQUET), Default::default())?
+        let lf = LazyFrame::scan_parquet(PlRefPath::new(FOODS_PARQUET), Default::default())?
             .with_row_index("index", Some(offset));
         assert!(row_index_at_scan(lf.clone()));
         let df = lf.collect()?;
@@ -656,7 +658,7 @@ fn test_row_index_on_files() -> PolarsResult<()> {
         );
 
         let lf = LazyFrame::scan_ipc(
-            PlPath::new(FOODS_IPC),
+            PlRefPath::new(FOODS_IPC),
             Default::default(),
             Default::default(),
         )?
@@ -683,7 +685,7 @@ fn test_row_index_on_files() -> PolarsResult<()> {
 
 #[test]
 fn scan_predicate_on_set_null_values() -> PolarsResult<()> {
-    let df = LazyCsvReader::new(PlPath::new(FOODS_CSV))
+    let df = LazyCsvReader::new(PlRefPath::new(FOODS_CSV))
         .with_null_values(Some(NullValues::Named(vec![("fats_g".into(), "0".into())])))
         .with_infer_schema_length(Some(0))
         .finish()?
@@ -792,7 +794,7 @@ fn scan_small_dtypes() -> PolarsResult<()> {
         DataType::UInt16,
     ];
     for dt in small_dt {
-        let df = LazyCsvReader::new(PlPath::new(FOODS_CSV))
+        let df = LazyCsvReader::new(PlRefPath::new(FOODS_CSV))
             .with_has_header(true)
             .with_dtype_overwrite(Some(Arc::new(Schema::from_iter([Field::new(
                 "sugars_g".into(),
