@@ -14,7 +14,6 @@ mod projection;
 mod projection_simple;
 mod projection_utils;
 mod scan;
-mod sink;
 mod slice;
 mod sort;
 mod stack;
@@ -46,7 +45,6 @@ pub(super) use self::merge_sorted::*;
 pub(super) use self::projection::*;
 pub(super) use self::projection_simple::*;
 pub(super) use self::scan::*;
-pub(super) use self::sink::*;
 pub(super) use self::slice::*;
 pub(super) use self::sort::*;
 pub(super) use self::stack::*;
