@@ -113,6 +113,8 @@ fn aexpr_to_skip_batch_predicate_rec(
             AExpr::Element => None,
             AExpr::Explode { .. } => None,
             AExpr::Column(_) => None,
+            #[cfg(feature = "dtype-struct")]
+            AExpr::StructField(_) => None,
             AExpr::Literal(_) => None,
             AExpr::BinaryExpr { left, op, right } => {
                 let left = *left;
@@ -297,6 +299,8 @@ fn aexpr_to_skip_batch_predicate_rec(
             AExpr::Ternary { .. } => None,
             AExpr::AnonymousFunction { .. } => None,
             AExpr::Eval { .. } => None,
+            #[cfg(feature = "dtype-struct")]
+            AExpr::StructEval { .. } => None,
             AExpr::Function {
                 input, function, ..
             } => match function {
