@@ -12,6 +12,8 @@ pub fn node_to_expr(node: Node, expr_arena: &Arena<AExpr>) -> Expr {
             options,
         },
         AExpr::Column(a) => Expr::Column(a),
+        #[cfg(feature = "dtype-struct")]
+        AExpr::StructField(a) => Expr::Field(Arc::new([a])),
         AExpr::Literal(s) => Expr::Literal(s),
         AExpr::BinaryExpr { left, op, right } => {
             let l = node_to_expr(left, expr_arena);
@@ -45,6 +47,7 @@ pub fn node_to_expr(node: Node, expr_arena: &Arena<AExpr>) -> Expr {
             expr,
             idx,
             returns_scalar,
+            null_on_oob,
         } => {
             let expr = node_to_expr(expr, expr_arena);
             let idx = node_to_expr(idx, expr_arena);
@@ -52,6 +55,7 @@ pub fn node_to_expr(node: Node, expr_arena: &Arena<AExpr>) -> Expr {
                 expr: Arc::new(expr),
                 idx: Arc::new(idx),
                 returns_scalar,
+                null_on_oob,
             }
         },
         AExpr::SortBy {
@@ -101,6 +105,25 @@ pub fn node_to_expr(node: Node, expr_arena: &Arena<AExpr>) -> Expr {
                 }
                 .into()
             },
+            IRAggExpr::MinBy { input, by } => {
+                let input_exp = node_to_expr(input, expr_arena);
+                let by_exp = node_to_expr(by, expr_arena);
+                AggExpr::MinBy {
+                    input: Arc::new(input_exp),
+                    by: Arc::new(by_exp),
+                }
+                .into()
+            },
+
+            IRAggExpr::MaxBy { input, by } => {
+                let input_exp = node_to_expr(input, expr_arena);
+                let by_exp = node_to_expr(by, expr_arena);
+                AggExpr::MaxBy {
+                    input: Arc::new(input_exp),
+                    by: Arc::new(by_exp),
+                }
+                .into()
+            },
 
             IRAggExpr::Mean(expr) => {
                 let exp = node_to_expr(expr, expr_arena);
@@ -219,6 +242,11 @@ pub fn node_to_expr(node: Node, expr_arena: &Arena<AExpr>) -> Expr {
             evaluation: Arc::new(node_to_expr(evaluation, expr_arena)),
             variant,
         },
+        #[cfg(feature = "dtype-struct")]
+        AExpr::StructEval { expr, evaluation } => Expr::StructEval {
+            expr: Arc::new(node_to_expr(expr, expr_arena)),
+            evaluation: expr_irs_to_exprs(evaluation, expr_arena),
+        },
         AExpr::Function {
             input,
             function,
@@ -557,7 +585,6 @@ pub fn ir_function_to_dsl(input: Vec<Expr>, function: IRFunctionExpr) -> Expr {
                 IB::SuffixFields(pl_small_str) => B::SuffixFields(pl_small_str),
                 #[cfg(feature = "json")]
                 IB::JsonEncode => B::JsonEncode,
-                IB::WithFields => B::WithFields,
                 IB::MapFieldNames(f) => B::MapFieldNames(f),
             })
         },
