@@ -30,7 +30,7 @@ impl PyDataFrame {
         }
         PyTuple::new(
             py,
-            df.get_columns().iter().map(|s| match s.dtype() {
+            df.columns().iter().map(|s| match s.dtype() {
                 DataType::Object(_) => {
                     let obj: Option<&ObjectValue> = s.get_object(idx).map(|any| any.into());
                     obj.into_py_any(py).unwrap()
@@ -48,7 +48,7 @@ impl PyDataFrame {
         // TODO: iterate over the chunks directly instead of using random access.
         let df = if df.max_n_chunks() > 16 {
             rechunked = df.clone();
-            py.enter_polars_ok(|| rechunked.as_single_chunk_par())?;
+            py.enter_polars_ok(|| rechunked.rechunk_mut_par())?;
             &rechunked
         } else {
             &df
@@ -58,7 +58,7 @@ impl PyDataFrame {
             (0..df.height()).map(|idx| {
                 PyTuple::new(
                     py,
-                    df.get_columns().iter().map(|c| match c.dtype() {
+                    df.columns().iter().map(|c| match c.dtype() {
                         DataType::Null => py.None(),
                         DataType::Object(_) => {
                             let obj: Option<&ObjectValue> = c.get_object(idx).map(|any| any.into());
@@ -109,12 +109,12 @@ impl PyDataFrame {
     pub fn to_pandas(&self, py: Python) -> PyResult<Vec<Py<PyAny>>> {
         let mut df = self.df.write();
         let dfr = &mut *df; // Lock guard isn't Send, but mut ref is.
-        py.enter_polars_ok(|| dfr.as_single_chunk_par())?;
+        py.enter_polars_ok(|| dfr.rechunk_mut_par())?;
         let df = RwLockWriteGuard::downgrade(df);
         Python::attach(|py| {
             let pyarrow = py.import("pyarrow")?;
             let cat_columns = df
-                .get_columns()
+                .columns()
                 .iter()
                 .enumerate()
                 .filter(|(_i, s)| {
@@ -180,7 +180,7 @@ impl PyDataFrame {
         requested_schema: Option<Py<PyAny>>,
     ) -> PyResult<Bound<'py, PyCapsule>> {
         py.enter_polars_ok(|| {
-            self.df.write().as_single_chunk_par();
+            self.df.write().rechunk_mut_par();
         })?;
         dataframe_to_stream(&self.df.read(), py)
     }
