@@ -1,5 +1,6 @@
 use arrow::array::Array;
 use polars::prelude::*;
+use polars_core::with_match_physical_numeric_polars_type;
 use pyo3::prelude::*;
 
 use super::PySeries;
@@ -9,7 +10,7 @@ use crate::utils::EnterPolarsExt;
 impl PySeries {
     fn scatter(&self, py: Python<'_>, idx: PySeries, values: PySeries) -> PyResult<()> {
         py.enter_polars(|| {
-            // we take the value because we want a ref count of 1 so that we can
+            // We take the value because we want a ref count of 1 so that we can
             // have mutable access cheaply via _get_inner_mut().
             let mut lock = self.series.write();
             let s = std::mem::take(&mut *lock);
@@ -20,8 +21,7 @@ impl PySeries {
                     Ok(())
                 },
                 Err((s, e)) => {
-                    // Restore original series:
-                    *lock = s;
+                    *lock = s; // Restore original series.
                     Err(e)
                 },
             }
@@ -29,16 +29,30 @@ impl PySeries {
     }
 }
 
-fn scatter(mut s: Series, idx: &Series, values: &Series) -> Result<Series, (Series, PolarsError)> {
+fn scatter(s: Series, idx: &Series, values: &Series) -> Result<Series, (Series, PolarsError)> {
     let logical_dtype = s.dtype().clone();
-
+    let converted_values;
     let values = if logical_dtype.is_categorical() || logical_dtype.is_enum() {
         if matches!(
             values.dtype(),
             DataType::Categorical(_, _) | DataType::Enum(_, _) | DataType::String | DataType::Null
         ) {
-            match values.strict_cast(&logical_dtype) {
-                Ok(values) => values,
+            converted_values = values.strict_cast(&logical_dtype);
+            match converted_values {
+                Ok(ref values) => values,
+                Err(err) => return Err((s, err)),
+            }
+        } else {
+            return Err((
+                s,
+                polars_err!(InvalidOperation: "invalid values dtype '{}' for scattering into dtype '{}'", values.dtype(), logical_dtype),
+            ));
+        }
+    } else if logical_dtype.is_decimal() {
+        if values.dtype().is_numeric() {
+            converted_values = values.strict_cast(&logical_dtype);
+            match converted_values {
+                Ok(ref values) => values,
                 Err(err) => return Err((s, err)),
             }
         } else {
@@ -48,23 +62,22 @@ fn scatter(mut s: Series, idx: &Series, values: &Series) -> Result<Series, (Seri
             ));
         }
     } else {
-        values.clone()
+        values
     };
 
-    let idx = match polars_ops::prelude::convert_to_unsigned_index(idx, s.len()) {
+    let null_on_oob = false;
+    let idx = match polars_ops::prelude::convert_to_unsigned_index(idx, s.len(), null_on_oob) {
         Ok(idx) => idx,
         Err(err) => return Err((s, err)),
     };
     let idx = idx.rechunk();
     let idx = idx.downcast_as_array();
-
-    if idx.null_count() > 0 {
+    if idx.has_nulls() {
         return Err((
             s,
             PolarsError::ComputeError("index values should not be null".into()),
         ));
     }
-
     let idx = idx.values().as_slice();
 
     let mut values = match values.to_physical_repr().cast(&s.dtype().to_physical()) {
@@ -72,93 +85,56 @@ fn scatter(mut s: Series, idx: &Series, values: &Series) -> Result<Series, (Seri
         Err(err) => return Err((s, err)),
     };
 
-    // Broadcast values input
+    // Broadcast values input.
     if values.len() == 1 && idx.len() > 1 {
         values = values.new_from_index(0, idx.len());
     }
 
-    // do not shadow, otherwise s is not dropped immediately
-    // and we want to have mutable access
-    s = s.to_physical_repr().into_owned();
-    let s_mut_ref = &mut s;
-    scatter_impl(s_mut_ref, logical_dtype, idx, &values).map_err(|err| (s, err))
+    let mut phys = s.to_physical_repr().into_owned();
+    drop(s); // Reduce refcount to make use of in-place mutation of possible.
+    let ret = scatter_impl(&mut phys, &logical_dtype, idx, &values);
+    match ret {
+        Ok(s) => Ok(unsafe { s.from_physical_unchecked(&logical_dtype).unwrap() }),
+        Err(e) => Err((
+            unsafe { phys.from_physical_unchecked(&logical_dtype).unwrap() },
+            e,
+        )),
+    }
 }
 
 fn scatter_impl(
     s: &mut Series,
-    logical_dtype: DataType,
+    logical_dtype: &DataType,
     idx: &[IdxSize],
     values: &Series,
 ) -> PolarsResult<Series> {
     let mutable_s = s._get_inner_mut();
 
-    let s = match logical_dtype.to_physical() {
-        DataType::Int8 => {
-            let ca: &mut ChunkedArray<Int8Type> = mutable_s.as_mut();
-            let values = values.i8()?;
-            ca.scatter(idx, values)
-        },
-        DataType::Int16 => {
-            let ca: &mut ChunkedArray<Int16Type> = mutable_s.as_mut();
-            let values = values.i16()?;
-            ca.scatter(idx, values)
-        },
-        DataType::Int32 => {
-            let ca: &mut ChunkedArray<Int32Type> = mutable_s.as_mut();
-            let values = values.i32()?;
-            ca.scatter(idx, values)
-        },
-        DataType::Int64 => {
-            let ca: &mut ChunkedArray<Int64Type> = mutable_s.as_mut();
-            let values = values.i64()?;
-            ca.scatter(idx, values)
-        },
-        DataType::UInt8 => {
-            let ca: &mut ChunkedArray<UInt8Type> = mutable_s.as_mut();
-            let values = values.u8()?;
-            ca.scatter(idx, values)
-        },
-        DataType::UInt16 => {
-            let ca: &mut ChunkedArray<UInt16Type> = mutable_s.as_mut();
-            let values = values.u16()?;
-            ca.scatter(idx, values)
-        },
-        DataType::UInt32 => {
-            let ca: &mut ChunkedArray<UInt32Type> = mutable_s.as_mut();
-            let values = values.u32()?;
-            ca.scatter(idx, values)
-        },
-        DataType::UInt64 => {
-            let ca: &mut ChunkedArray<UInt64Type> = mutable_s.as_mut();
-            let values = values.u64()?;
-            ca.scatter(idx, values)
-        },
-        DataType::Float32 => {
-            let ca: &mut ChunkedArray<Float32Type> = mutable_s.as_mut();
-            let values = values.f32()?;
-            ca.scatter(idx, values)
-        },
-        DataType::Float64 => {
-            let ca: &mut ChunkedArray<Float64Type> = mutable_s.as_mut();
-            let values = values.f64()?;
-            ca.scatter(idx, values)
+    match mutable_s.dtype() {
+        dt if dt.is_primitive_numeric() => {
+            with_match_physical_numeric_polars_type!(dt, |$T| {
+                let ca: &mut ChunkedArray<$T> = mutable_s.as_mut();
+                let values: &ChunkedArray<$T> = values.as_ref().as_ref();
+                ca.scatter(idx, values)
+            })
         },
         DataType::Boolean => {
-            let ca = s.bool()?;
+            let ca: &mut ChunkedArray<BooleanType> = mutable_s.as_mut();
             let values = values.bool()?;
             ca.scatter(idx, values)
         },
+        DataType::Binary => {
+            let ca: &mut ChunkedArray<BinaryType> = mutable_s.as_mut();
+            let values = values.binary()?;
+            ca.scatter(idx, values)
+        },
         DataType::String => {
-            let ca = s.str()?;
+            let ca: &mut ChunkedArray<StringType> = mutable_s.as_mut();
             let values = values.str()?;
             ca.scatter(idx, values)
         },
-        _ => {
-            return Err(PolarsError::ComputeError(
-                format!("not yet implemented for dtype: {logical_dtype}").into(),
-            ));
-        },
-    };
-
-    s.and_then(|s| s.cast(&logical_dtype))
+        _ => Err(PolarsError::ComputeError(
+            format!("not yet implemented for dtype: {logical_dtype}").into(),
+        )),
+    }
 }
