@@ -11,8 +11,8 @@ use polars_io::cloud::CloudOptions;
 use polars_plan::dsl::deletion::DeletionFilesList;
 use polars_plan::dsl::{CastColumnsPolicy, ScanSource};
 use polars_utils::format_pl_smallstr;
+use polars_utils::pl_path::PlRefPath;
 use polars_utils::pl_str::PlSmallStr;
-use polars_utils::plpath::PlPath;
 use polars_utils::relaxed_cell::RelaxedCell;
 use polars_utils::slice_enum::Slice;
 
@@ -109,7 +109,7 @@ impl DeletionFilesProvider {
                     .iter()
                     .enumerate()
                     .map(|(deletion_file_idx, path)| {
-                        let source = ScanSource::Path(PlPath::new(path));
+                        let source = ScanSource::Path(PlRefPath::new(path));
                         let mut reader = reader_builder.build_file_reader(
                             source,
                             cloud_options.clone(),
