@@ -616,6 +616,7 @@ def test_scan_nonexistent_path(format: str) -> None:
     "streaming",
     [True, False],
 )
+@pytest.mark.skipif(sys.platform == "win32", reason="Windows paths are different")
 def test_scan_include_file_paths(
     tmp_path: Path,
     scan_func: Callable[..., pl.LazyFrame],
@@ -1143,7 +1144,7 @@ def test_scan_empty_paths_friendly_error(
 
     assert (
         f"ComputeError: {failed_message}: expanded paths were empty "
-        "(path expansion input: 'paths: [Local"
+        "(path expansion input: 'paths: "
     ) in exc_str
 
     assert "glob: true)." in exc_str
@@ -1172,7 +1173,7 @@ def test_scan_empty_paths_friendly_error(
 
     assert (
         f"ComputeError: {failed_message}: expanded paths were empty "
-        "(path expansion input: 'paths: [Local"
+        "(path expansion input: 'paths: "
     ) in exc_str
 
     assert "glob: true)." in exc_str
