@@ -4,13 +4,14 @@ import json
 import sys
 import time
 from functools import lru_cache, partial
-from typing import TYPE_CHECKING, Any, Callable
+from typing import TYPE_CHECKING, Any
 
 import polars as pl
 from polars._plr import _ir_nodes
 from polars._utils.wrap import wrap_df
 
 if TYPE_CHECKING:
+    from collections.abc import Callable
     from pathlib import Path
 
     import pandas as pd
