@@ -17,6 +17,14 @@ def test_shift() -> None:
     assert_series_equal(a.shift(-1, fill_value=10), pl.Series("a", [2, 3, 10]))
 
 
+def test_shift_object() -> None:
+    a = pl.Series("a", [1, 2, 3], dtype=pl.Object)
+    assert a.shift(1).to_list() == [None, 1, 2]
+    assert a.shift(-1).to_list() == [2, 3, None]
+    assert a.shift(-2, fill_value=pl.lit(0, dtype=pl.Object)).to_list() == [3, 0, 0]
+    assert a.shift(1).dtype == pl.Object
+
+
 def test_shift_frame(fruits_cars: pl.DataFrame) -> None:
     df = pl.DataFrame({"a": [1, 2, 3, 4, 5], "b": [1, 2, 3, 4, 5]})
     out = df.select(pl.col("a").shift(1))
